package main

import (
	"encoding/gob"
	"log"
	"net/http"
	"os"

	"streifling.com/jason/cpolis/cmd/control"
	"streifling.com/jason/cpolis/cmd/model"
	"streifling.com/jason/cpolis/cmd/view"
)

func init() {
	gob.Register(model.User{})
}

func main() {
	args, err := control.HandleCliArgs()
	if err != nil {
		log.Fatalln(err)
	}

	logFile, err := os.OpenFile(args.LogFile, os.O_WRONLY|os.O_CREATE|os.O_APPEND, 0644)
	if err != nil {
		log.Fatalln(err)
	}
	defer logFile.Close()
	log.SetOutput(logFile)

	db, err := model.OpenDB(args.DBName)
	if err != nil {
		log.Fatalln(err)
	}
	defer db.Close()

	key, err := control.LoadKey(args.KeyFile)
	if err != nil {
		key, err = control.NewKey()
		if err != nil {
			log.Fatalln(err)
		}
		control.SaveKey(key, args.KeyFile)
	}
	store := control.NewCookieStore(key)

	mux := http.NewServeMux()
	mux.Handle("/web/static/", http.StripPrefix("/web/static/",
		http.FileServer(http.Dir(args.WebDir+"/static/"))))
	mux.HandleFunc("/", view.HomePage(args, db, store))

	mux.HandleFunc("GET /create-tag", view.CreateTag(args))
	mux.HandleFunc("GET /create-user", view.CreateUser(args))
	mux.HandleFunc("GET /edit-user", view.EditUser(args, db, store))
	mux.HandleFunc("GET /hub", view.ShowHub(args, db, store))
	mux.HandleFunc("GET /logout", view.Logout(args, store))
	mux.HandleFunc("GET /publish-article/{id}", view.PublishArticle(args, db, store))
	mux.HandleFunc("GET /publish-issue", view.PublishLatestIssue(args, db, store))
	mux.HandleFunc("GET /reject-article/{id}", view.RejectArticle(args, db, store))
	mux.HandleFunc("GET /rejected-articles", view.ShowRejectedArticles(args, db, store))
	mux.HandleFunc("GET /review-rejected-article/{id}", view.ReviewRejectedArticle(args, db, store))
	mux.HandleFunc("GET /review-unpublished-article/{id}", view.ReviewUnpublishedArticle(args, db, store))
	mux.HandleFunc("GET /rss", view.ShowRSS(
		args,
		db,
		"Freimaurer Distrikt Niedersachsen und Sachsen-Anhalt",
		"https://distrikt-ni-st.de",
		"Freiheit, Gleichheit, Brüderlichkeit, Toleranz und Humanität",
	))
	mux.HandleFunc("GET /this-issue", view.ShowCurrentArticles(args, db))
	mux.HandleFunc("GET /unpublished-articles", view.ShowUnpublishedArticles(args, db))
	mux.HandleFunc("GET /write-article", view.WriteArticle(args, db))

	mux.HandleFunc("POST /add-first-user", view.AddFirstUser(args, db, store))
	mux.HandleFunc("POST /add-tag", view.AddTag(args, db, store))
	mux.HandleFunc("POST /add-user", view.AddUser(args, db, store))
	mux.HandleFunc("POST /login", view.Login(args, db, store))
	mux.HandleFunc("POST /resubmit-article/{id}", view.ResubmitArticle(args, db, store))
	mux.HandleFunc("POST /submit-article", view.SubmitArticle(args, db, store))
	mux.HandleFunc("POST /update-user", view.UpdateUser(args, db, store))
	mux.HandleFunc("POST /upload-image", view.UploadImage(args))

	log.Fatalln(http.ListenAndServe(args.Port, mux))
}
